# Copyright (C) 2010, 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of LAVA Server.
#
# LAVA Server is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA Server is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA Server.  If not, see <http://www.gnu.org/licenses/>.

"""
Headless model application experiment
"""

from django.db import models
from django_restricted_resource.models  import RestrictedResource


class Project(RestrictedResource):
    """
    Project, container of everything else. Projects are restricted resources
    and thus belong to a particular user or group and have a "public" flag.
    Projects also allow for simple nesting, where one project can be a part
    of another.
    """

    name = models.CharField(
        null=False,
        blank=False,
        max_length=100)

    part_of = models.ForeignKey(
        "self",
        related_name="sub_projects",
        blank=True,
        null=True)

    is_aggregate = models.BooleanField(
        blank=True,
        null=False,
        help="Indicator for distribution-like projects")

    def __unicode__(self):
        return self.name


class Release(models.Model):
    """
    A snapshot of a project with a unique name.

    Releases are used to coordinate testing and help with project management. A
    release is "done" once released_date is set. After that it should not be
    changed any further.
    """
    
    project = models.ForeignKey(
        Project,
        null=False,
        blank=False,
        related_name="releases",
        help="Release to project association")

    name = models.CharField(
        null=False,
        blank=True,
        help="Release name, if any")

    expected_date = models.DateField(
        null=False,
        blank=False,
        help="When this release is expected to happen")

    released_date = models.DataField(
        null=True,
        blank=True,
        help="When this release has actually happened")


class Build(models.Model):
    """
    Build is the result of building the collective set of source code of a
    particular project. Builds are nothing special and there can be numerous
    builds can be made. Only the build name and date are currently recorded.
    Name is arbitrary but is expected to be generated by the build tool.
    """

    project = models.ForeignKey(
        Project,
        null=False,
        related_name="images")
    
    name = models.CharField(
        blank=True,
        max_length=1024,
        null=False)

    build_date = models.DateTimeField(
        null=False)

    def __unicode__(self):
        return self.name


class BuildArtifact(models.Model):
    """
    Build artifact is the tangible leftover of a build. Each artifact has a
    name and an URL. The name is usually the basename of the URL. To organize
    artifacts sensibly a project can define a set of artifact types.
    """

    name = models.CharField(
        blank=True,
        max_length=1024,
        null=False)

    url = models.CharField(
        blank=False,
        max_length=1024,
        null=False)

    build = models.ForeignKey(
        Build,
        related_name="artifacts")

    type = models.ForeignKey(
        "BuildArtifactType",
        null=False,
        related_name="artifacts")

    def __unicode__(self):
        return self.name


class SystemImage(models.Model):
    """
    System image is a special extension of a build that is suitable as an
    operating system image.
    """

    release = models.ForeignKey(
        Relase,
        null=True,
        blank=True,
        related_name="system_images")

    build = models.ForeignKey(
        Build,
        null=False,
        blank=False,
        related_name="system_images")


class ReleaseManualTesting(models.Model):

    release = models.ForeignKey(
        Release,
        unique=True,
        related_name="manual_testing_set",
        null=False,
        blank=False)

    testing_scenario_branch = models.CharField(
        null=False,
        blank=True,
        max_length=512,
        help="Bazaar URL for the branch that defines manual testing scenarios")

    testing_scenario_revision = models.CharField(
        null=False,
        blank=True,
        max_length=512,
        help="Identifier of the revision that should be used for this release")

    stream = models.ForeignKey(
        "dashboard_app.BundleStream",
        null=False,
        blank=False,
        related_name="release_manual_testing_plans")
        help="Bundle stream to which manual test results are submitted")






class BuildArtifactType(models.Model):
    """
    A classifier that simplifies making sense of build artifacts. Build
    artifact type is specific to a particular project. 
    """

    name = models.CharField(
        null=False,
        blank=False,
        max_length=100)

    project = models.ForeignKey(
        Project,
        null=False,
        related_name="build_artifact_types")

    # TODO: It would be nice to be able to have artifact types that belong to a
    # parent project (or parent of the parent project). Then we could have a
    # "sharing type" field that defines if sub projects can use this artifact
    # type.

    def __unicode__(self):
        return self.name
